require 'tpt/rails/internal/datadog'
require 'tpt/rails/internal/error_reporter'
require 'tpt/rails/internal/health_checks'

class Tpt::Rails::Config
  #
  # You can set the following configs in your `Tpt::Rails.configure do |config|` block:
  #

  # E.g. production/integration/jordan123
  attr_accessor :app_env
  # E.g. earnings/resource-write/tpt-data
  attr_accessor :app_name
  # A project-specific api key from Bugsnag
  attr_accessor :bugsnag_api_key
  # Set this in the form of: statsd://[host]:[port]'
  attr_accessor :datadog_statsd_url
  # Set this in the form of: datadog://[host]:[port]'
  attr_accessor :datadog_trace_url
  # Set this in the form of: redis://:[password]@[hostname]:[port]/[db]
  attr_accessor :redis_url
  # A project-specific access token from rollbar
  attr_accessor :rollbar_access_token
  # Allow enabling/disabling Rollbar. Defaults to false.
  attr_accessor :rollbar_enabled

  # Add a health check to the endpoint provided at `/internal/health-check` by
  # Tpt::Rails::HealthChecksController.
  #
  # The block provided to this method should return a truthy/falsey value for success/failure.
  #
  # Example:
  #
  #   config.health_check(:foo) { 'foo' == 'foo' }
  #
  def health_check(name, &block)
    @health_checks.add(name, &block)
  end

  #
  # Internal code
  #

  attr_reader :health_checks # :nodoc:

  def initialize # :nodoc:
    @health_checks = Tpt::Rails::Internal::HealthChecks.new
  end

  def setup # :nodoc:
    if redis_url
      # Ensure that redis has been added to the app's Gemfile if a `redis_url` has been set
      gem 'redis', '>= 4', '< 5'
      gem 'hiredis'

      Redis.current = Redis.new(url: redis_url, driver: :hiredis)
    end

    @datadog = Tpt::Rails::Internal::Datadog.new(
      statsd_url: datadog_statsd_url,
      trace_url: datadog_trace_url,
    )

    @health_checks.setup

    @error_reporter = Tpt::Rails::Internal::ErrorReporter.new(
      bugsnag_api_key: bugsnag_api_key.presence,
      rollbar_access_token: rollbar_access_token.presence,
      rollbar_enabled: rollbar_enabled ? true : false,
    )
  end

  def statsd # :nodoc:
    @datadog.statsd
  end

  def report(error) # :nodoc:
    @error_reporter.report(error)
  end
end
