class Tpt::Rails::Internal::ErrorReporter # :nodoc:
  def initialize(bugsnag_api_key: nil, rollbar_access_token: nil, rollbar_enabled: false)
    @bugsnag_api_key = bugsnag_api_key
    @rollbar_access_token = rollbar_access_token
    @rollbar_enabled = rollbar_enabled

    configure_bugsnag if bugsnag?
    configure_rollbar if rollbar?
  end

  def report(error)
    if bugsnag?
      Bugsnag.notify(error)
    end

    if rollbar?
      Rollbar.error(error)
    end

    Tpt::Rails.statsd.increment(
      'error',
      tags: [
        "error_class:#{error.class.name.underscore}",
        'type:reported',
      ],
    )

    ::Rails.logger.info("Error: class=#{error.class} message=#{error.message.inspect}")

    if !bugsnag? && !rollbar?
      ::Rails.logger.info('Warning: No error reporter configured for Tpt::Rails. Please add one.')
      ::Rails.logger.info(error.backtrace)
    end
  end

  private

  def bugsnag?
    @bugsnag_api_key
  end

  def rollbar?
    @rollbar_access_token
  end

  def configure_bugsnag
    # Ensure that bugsnag has been added to the app's Gemfile
    gem 'bugsnag', '>= 6.12.2', '< 7'

    Bugsnag.configure do |config|
      config.api_key = @bugsnag_api_key
      config.app_version = Tpt::Rails.release_version if Tpt::Rails.release_version
    end
  end

  def configure_rollbar
    # Ensure that rollbar has been added to the app's Gemfile
    gem 'rollbar', '>= 2.23.2', '< 4'

    Rollbar.configure do |config|
      config.access_token = @rollbar_access_token
      config.code_version = Tpt::Rails.release_version if Tpt::Rails.release_version
      config.enabled = @rollbar_enabled
      config.environment = Tpt::Rails.app_env
      config.populate_empty_backtraces = true
    end
  end
end
